==> opensaml-saml-impl-3.4.2-sources.jar: org/opensaml/saml/saml2/binding/encoding/impl/HTTPPostSimpleSignEncoder.java <==
/*
 * Licensed to the University Corporation for Advanced Internet Development,
 * Inc. (UCAID) under one or more contributor license agreements.  See the
 * NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The UCAID licenses this file to You under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opensaml.saml.saml2.binding.encoding.impl;

import java.io.UnsupportedEncodingException;

import net.shibboleth.utilities.java.support.codec.Base64Support;
import net.shibboleth.utilities.java.support.xml.SerializeSupport;

import org.apache.velocity.VelocityContext;
import org.opensaml.core.xml.config.XMLObjectProviderRegistrySupport;
import org.opensaml.core.xml.io.Marshaller;
import org.opensaml.core.xml.io.MarshallingException;
import org.opensaml.messaging.context.MessageContext;
import org.opensaml.messaging.encoder.MessageEncodingException;
import org.opensaml.saml.common.SAMLObject;
import org.opensaml.saml.common.binding.SAMLBindingSupport;
import org.opensaml.saml.common.messaging.SAMLMessageSecuritySupport;
import org.opensaml.saml.common.xml.SAMLConstants;
import org.opensaml.security.SecurityException;
import org.opensaml.security.credential.Credential;
import org.opensaml.security.credential.CredentialSupport;
import org.opensaml.xmlsec.SignatureSigningParameters;
import org.opensaml.xmlsec.crypto.XMLSigningUtil;
import org.opensaml.xmlsec.keyinfo.KeyInfoGenerator;
import org.opensaml.xmlsec.signature.KeyInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.base.Strings;

/**
 * SAML 2.0 HTTP-POST-SimpleSign binding message encoder.
 */
public class HTTPPostSimpleSignEncoder extends HTTPPostEncoder {
    
    /** Default template ID. */
    public static final String DEFAULT_TEMPLATE_ID = "/templates/saml2-post-simplesign-binding.vm";

    /** Class logger. */
    private final Logger log = LoggerFactory.getLogger(HTTPPostSimpleSignEncoder.class);
    
    /** Constructor. */
    public HTTPPostSimpleSignEncoder() {
        setVelocityTemplateId(DEFAULT_TEMPLATE_ID);
    }

    /** {@inheritDoc} */
    public String getBindingURI() {
        return SAMLConstants.SAML2_POST_SIMPLE_SIGN_BINDING_URI;
    }

    /** {@inheritDoc} */
    protected void populateVelocityContext(final VelocityContext velocityContext,
            final MessageContext<SAMLObject> messageContext,
            final String endpointURL) throws MessageEncodingException {

        super.populateVelocityContext(velocityContext, messageContext, endpointURL);

        final SignatureSigningParameters signingParameters = 
                SAMLMessageSecuritySupport.getContextSigningParameters(messageContext);
        
        if (signingParameters == null || signingParameters.getSigningCredential() == null) {
            log.debug("No signing credential was supplied, skipping HTTP-Post simple signing");
            return;
        }

        final String sigAlgURI = getSignatureAlgorithmURI(signingParameters);
        velocityContext.put("SigAlg", sigAlgURI);

        final String formControlData = buildFormDataToSign(velocityContext, messageContext, sigAlgURI);
        velocityContext.put("Signature", generateSignature(signingParameters.getSigningCredential(), 
                sigAlgURI, formControlData));

        
        final KeyInfoGenerator kiGenerator = signingParameters.getKeyInfoGenerator();
        if (kiGenerator != null) {
            final String kiBase64 = buildKeyInfo(signingParameters.getSigningCredential(), kiGenerator);
            if (!Strings.isNullOrEmpty(kiBase64)) {
                velocityContext.put("KeyInfo", kiBase64);
            }
        }
    }

    /**
     * Build the {@link KeyInfo} from the signing credential.
     * 
     * @param signingCredential the credential used for signing
     * @param kiGenerator the generator for the KeyInfo
     * @throws MessageEncodingException thrown if there is an error generating or marshalling the KeyInfo
     * @return the marshalled, serialized and base64-encoded KeyInfo, or null if none was generated
     */
    protected String buildKeyInfo(final Credential signingCredential, final KeyInfoGenerator kiGenerator)
            throws MessageEncodingException {

        try {
            final KeyInfo keyInfo = kiGenerator.generate(signingCredential);
            if (keyInfo != null) {
                final Marshaller marshaller =
                        XMLObjectProviderRegistrySupport.getMarshallerFactory().getMarshaller(keyInfo);
                if (marshaller == null) {
                    log.error("No KeyInfo marshaller available from configuration");
                    throw new MessageEncodingException("No KeyInfo marshaller was configured");
                }
                final String kiXML = SerializeSupport.nodeToString(marshaller.marshall(keyInfo));
                final String kiBase64 = Base64Support.encode(kiXML.getBytes(), Base64Support.UNCHUNKED);
                return kiBase64;
            } else {
                return null;
            }
        } catch (final SecurityException e) {
            log.error("Error generating KeyInfo from signing credential", e);
            throw new MessageEncodingException("Error generating KeyInfo from signing credential", e);
        } catch (final MarshallingException e) {
            log.error("Error marshalling KeyInfo based on signing credential", e);
            throw new MessageEncodingException("Error marshalling KeyInfo based on signing credential", e);
        }
    }

    /**
     * Build the form control data string over which the signature is computed.
     * 
     * @param velocityContext the Velocity context which is already populated with the values for SAML message and relay
     *            state
     * @param messageContext the SAML message context being processed
     * @param sigAlgURI the signature algorithm URI
     * 
     * @return the form control data string for signature computation
     */
    protected String buildFormDataToSign(final VelocityContext velocityContext,
            final MessageContext<SAMLObject> messageContext,
            final String sigAlgURI) {
        final StringBuilder builder = new StringBuilder();

        boolean isRequest = false;
        if (velocityContext.get("SAMLRequest") != null) {
            isRequest = true;
        }

        final String msgB64;
        if (isRequest) {
            msgB64 = (String) velocityContext.get("SAMLRequest");
        } else {
            msgB64 = (String) velocityContext.get("SAMLResponse");
        }

        String msg = null;
        try {
            msg = new String(Base64Support.decode(msgB64), "UTF-8");
        } catch (final UnsupportedEncodingException e) {
            // All JVM's required to support UTF-8
        }

        if (isRequest) {
            builder.append("SAMLRequest=" + msg);
        } else {
            builder.append("SAMLResponse=" + msg);
        }

        final String relayState = SAMLBindingSupport.getRelayState(messageContext);
        if (relayState != null) {
            builder.append("&RelayState=" + relayState);
        }

        builder.append("&SigAlg=" + sigAlgURI);

        return builder.toString();
    }
    
    /**
     * Gets the signature algorithm URI to use.
     * 
     * @param signingParameters the signing parameters to use
     * 
     * @return signature algorithm to use with the associated signing credential
     * 
     * @throws MessageEncodingException thrown if the algorithm URI is not supplied explicitly and 
     *          could not be derived from the supplied credential
     */
    protected String getSignatureAlgorithmURI(final SignatureSigningParameters signingParameters)
            throws MessageEncodingException {
        
        if (signingParameters.getSignatureAlgorithm() != null) {
            return signingParameters.getSignatureAlgorithm();
        }

        throw new MessageEncodingException("The signing algorithm URI could not be determined");
    }

    /**
     * Generates the signature over the string of concatenated form control data as indicated by the SimpleSign spec.
     * 
     * @param signingCredential credential that will be used to sign
     * @param algorithmURI algorithm URI of the signing credential
     * @param formData form control data to be signed
     * 
     * @return base64 encoded signature of form control data
     * 
     * @throws MessageEncodingException there is an error computing the signature
     */
    protected String generateSignature(final Credential signingCredential, final String algorithmURI,
            final String formData)
            throws MessageEncodingException {

        log.debug(String.format(
                "Generating signature with key type '%s', algorithm URI '%s' over form control string '%s'",
                CredentialSupport.extractSigningKey(signingCredential).getAlgorithm(), algorithmURI, formData));

        String b64Signature = null;
        try {
            final byte[] rawSignature =
                    XMLSigningUtil.signWithURI(signingCredential, algorithmURI, formData.getBytes("UTF-8"));
            b64Signature = Base64Support.encode(rawSignature, Base64Support.UNCHUNKED);
            log.debug("Generated digital signature value (base64-encoded) {}", b64Signature);
        } catch (final SecurityException e) {
            log.error("Error during URL signing process", e);
            throw new MessageEncodingException("Unable to sign form control string", e);
        } catch (final UnsupportedEncodingException e) {
            // UTF-8 encoding is required to be supported by all JVMs
        }

        return b64Signature;
    }

}

==> opensaml-saml-impl-4.1.1-sources.jar: org/opensaml/saml/saml2/binding/encoding/impl/HTTPPostSimpleSignEncoder.java <==
/*
 * Licensed to the University Corporation for Advanced Internet Development,
 * Inc. (UCAID) under one or more contributor license agreements.  See the
 * NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The UCAID licenses this file to You under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opensaml.saml.saml2.binding.encoding.impl;

import java.io.UnsupportedEncodingException;

import net.shibboleth.utilities.java.support.codec.Base64Support;
import net.shibboleth.utilities.java.support.codec.DecodingException;
import net.shibboleth.utilities.java.support.codec.EncodingException;
import net.shibboleth.utilities.java.support.xml.SerializeSupport;

import org.apache.velocity.VelocityContext;
import org.opensaml.core.xml.config.XMLObjectProviderRegistrySupport;
import org.opensaml.core.xml.io.Marshaller;
import org.opensaml.core.xml.io.MarshallingException;
import org.opensaml.messaging.context.MessageContext;
import org.opensaml.messaging.encoder.MessageEncodingException;
import org.opensaml.saml.common.binding.SAMLBindingSupport;
import org.opensaml.saml.common.messaging.SAMLMessageSecuritySupport;
import org.opensaml.saml.common.xml.SAMLConstants;
import org.opensaml.security.SecurityException;
import org.opensaml.security.credential.Credential;
import org.opensaml.security.credential.CredentialSupport;
import org.opensaml.xmlsec.SignatureSigningParameters;
import org.opensaml.xmlsec.crypto.XMLSigningUtil;
import org.opensaml.xmlsec.keyinfo.KeyInfoGenerator;
import org.opensaml.xmlsec.signature.KeyInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.base.Strings;

/**
 * SAML 2.0 HTTP-POST-SimpleSign binding message encoder.
 */
public class HTTPPostSimpleSignEncoder extends HTTPPostEncoder {
    
    /** Default template ID. */
    public static final String DEFAULT_TEMPLATE_ID = "/templates/saml2-post-simplesign-binding.vm";

    /** Class logger. */
    private final Logger log = LoggerFactory.getLogger(HTTPPostSimpleSignEncoder.class);
    
    /** Constructor. */
    public HTTPPostSimpleSignEncoder() {
        setVelocityTemplateId(DEFAULT_TEMPLATE_ID);
    }

    /** {@inheritDoc} */
    public String getBindingURI() {
        return SAMLConstants.SAML2_POST_SIMPLE_SIGN_BINDING_URI;
    }

    /** {@inheritDoc} */
    protected void populateVelocityContext(final VelocityContext velocityContext, final MessageContext messageContext,
            final String endpointURL) throws MessageEncodingException {

        super.populateVelocityContext(velocityContext, messageContext, endpointURL);

        final SignatureSigningParameters signingParameters = 
                SAMLMessageSecuritySupport.getContextSigningParameters(messageContext);
        
        if (signingParameters == null || signingParameters.getSigningCredential() == null) {
            log.debug("No signing credential was supplied, skipping HTTP-Post simple signing");
            return;
        }

        final String sigAlgURI = getSignatureAlgorithmURI(signingParameters);
        velocityContext.put("SigAlg", sigAlgURI);

        final String formControlData = buildFormDataToSign(velocityContext, messageContext, sigAlgURI);
        velocityContext.put("Signature", generateSignature(signingParameters.getSigningCredential(), 
                sigAlgURI, formControlData));

        
        final KeyInfoGenerator kiGenerator = signingParameters.getKeyInfoGenerator();
        if (kiGenerator != null) {
            final String kiBase64 = buildKeyInfo(signingParameters.getSigningCredential(), kiGenerator);
            if (!Strings.isNullOrEmpty(kiBase64)) {
                velocityContext.put("KeyInfo", kiBase64);
            }
        }
    }

    /**
     * Build the {@link KeyInfo} from the signing credential.
     * 
     * @param signingCredential the credential used for signing
     * @param kiGenerator the generator for the KeyInfo
     * @throws MessageEncodingException thrown if there is an error generating or marshalling the KeyInfo
     * @return the marshalled, serialized and base64-encoded KeyInfo, or null if none was generated
     */
    protected String buildKeyInfo(final Credential signingCredential, final KeyInfoGenerator kiGenerator)
            throws MessageEncodingException {

        try {
            final KeyInfo keyInfo = kiGenerator.generate(signingCredential);
            if (keyInfo != null) {
                final Marshaller marshaller =
                        XMLObjectProviderRegistrySupport.getMarshallerFactory().getMarshaller(keyInfo);
                if (marshaller == null) {
                    log.error("No KeyInfo marshaller available from configuration");
                    throw new MessageEncodingException("No KeyInfo marshaller was configured");
                }
                final String kiXML = SerializeSupport.nodeToString(marshaller.marshall(keyInfo));
                final String kiBase64 = Base64Support.encode(kiXML.getBytes(), Base64Support.UNCHUNKED);
                return kiBase64;
            }
            return null;
        } catch (final SecurityException e) {
            log.error("Error generating KeyInfo from signing credential: {}", e.getMessage());
            throw new MessageEncodingException("Error generating KeyInfo from signing credential", e);
        } catch (final MarshallingException e) {
            log.error("Error marshalling KeyInfo based on signing credential: {}", e.getMessage());
            throw new MessageEncodingException("Error marshalling KeyInfo based on signing credential", e);
        } catch (final EncodingException e) {
            log.error("Error base64 encoding KeyInfo from signing credential: {}", e.getMessage());
            throw new MessageEncodingException("Error base64 encoding KeyInfo from signing credential", e);
        }
    }

    /**
     * Build the form control data string over which the signature is computed.
     * 
     * @param velocityContext the Velocity context which is already populated with the values for SAML message and relay
     *            state
     * @param messageContext the SAML message context being processed
     * @param sigAlgURI the signature algorithm URI
     * 
     * @throws MessageEncodingException if there is an issue building the form to sign. 
     * 
     * @return the form control data string for signature computation
     */
    protected String buildFormDataToSign(final VelocityContext velocityContext, final MessageContext messageContext,
            final String sigAlgURI) throws MessageEncodingException {
        final StringBuilder builder = new StringBuilder();

        boolean isRequest = false;
        if (velocityContext.get("SAMLRequest") != null) {
            isRequest = true;
        }

        final String msgB64;
        if (isRequest) {
            msgB64 = (String) velocityContext.get("SAMLRequest");
        } else {
            msgB64 = (String) velocityContext.get("SAMLResponse");
        }
       
        String msg = null;
        try {
            msg = new String(Base64Support.decode(msgB64), "UTF-8");
        } catch (final UnsupportedEncodingException e) {
            // Should never happen, all JVM's required to support UTF-8           
            throw new MessageEncodingException("Unable to decode message as string, UTF-8 encoding is not supported",e);
        } catch (final DecodingException e) { 
            // Should never happen, original message is controlled and built by the IdP.
            throw new MessageEncodingException("Error base64-decoding the "+
                                            (isRequest ? "SAMLRequest" : "SAMLResponse"),e);
        }

        if (isRequest) {
            builder.append("SAMLRequest=" + msg);
        } else {
            builder.append("SAMLResponse=" + msg);
        }

        final String relayState = SAMLBindingSupport.getRelayState(messageContext);
        if (relayState != null) {
            builder.append("&RelayState=" + relayState);
        }

        builder.append("&SigAlg=" + sigAlgURI);

        return builder.toString();
    }
    
    /**
     * Gets the signature algorithm URI to use.
     * 
     * @param signingParameters the signing parameters to use
     * 
     * @return signature algorithm to use with the associated signing credential
     * 
     * @throws MessageEncodingException thrown if the algorithm URI is not supplied explicitly and 
     *          could not be derived from the supplied credential
     */
    protected String getSignatureAlgorithmURI(final SignatureSigningParameters signingParameters)
            throws MessageEncodingException {
        
        if (signingParameters.getSignatureAlgorithm() != null) {
            return signingParameters.getSignatureAlgorithm();
        }

        throw new MessageEncodingException("The signing algorithm URI could not be determined");
    }

    /**
     * Generates the signature over the string of concatenated form control data as indicated by the SimpleSign spec.
     * 
     * @param signingCredential credential that will be used to sign
     * @param algorithmURI algorithm URI of the signing credential
     * @param formData form control data to be signed
     * 
     * @return base64 encoded signature of form control data
     * 
     * @throws MessageEncodingException there is an error computing the signature
     */
    protected String generateSignature(final Credential signingCredential, final String algorithmURI,
            final String formData)
            throws MessageEncodingException {

        log.debug(String.format(
                "Generating signature with key type '%s', algorithm URI '%s' over form control string '%s'",
                CredentialSupport.extractSigningKey(signingCredential).getAlgorithm(), algorithmURI, formData));

        String b64Signature = null;
        try {
            final byte[] rawSignature =
                    XMLSigningUtil.signWithURI(signingCredential, algorithmURI, formData.getBytes("UTF-8"));
            b64Signature = Base64Support.encode(rawSignature, Base64Support.UNCHUNKED);
            log.debug("Generated digital signature value (base64-encoded) {}", b64Signature);
        } catch (final SecurityException e) {
            log.error("Error during URL signing process: {}", e.getMessage());
            throw new MessageEncodingException("Unable to sign form control string", e);
        } catch (final UnsupportedEncodingException e) {
            log.error("UTF-8 encoding is not supported, this VM is not Java compliant");
            throw new MessageEncodingException("Unable to encode message, UTF-8 encoding is not supported");
        } catch (final EncodingException e) {
            log.error("Error base64 encoding signature of form control data: {}",e.getMessage());
            throw new MessageEncodingException("Unable to base64 encode signature of form control data",e);
        }

        return b64Signature;
    }

}
